@@ -1,3 +1,5 @@
+from typing import Type
+
 from squyrrel.db.connection import SqlDatabaseConnection
 
 from squyrrel.orm.entity_format import EntityFormat
@@ -5,21 +7,22 @@ from squyrrel.orm.exceptions import *
 from squyrrel.orm.field import (ManyToOne, ManyToMany, StringField,
                                 DateTimeField, IntegerField)
 from squyrrel.orm.filter import (ManyToOneFilter, ManyToManyFilter)
+from squyrrel.orm.model import Model
 from squyrrel.orm.signals import model_loaded_signal
-from squyrrel.orm.utils import sanitize_id_array
-from squyrrel.orm.query_builder import QueryBuilder
+from squyrrel.orm.utils import sanitize_id_array, m2m_aggregation_subquery_alias
 from squyrrel.sql.query import (Query, UpdateQuery, InsertQuery,
                                 DeleteQuery, CreateTableQuery)
 from squyrrel.sql.clauses import *
 from squyrrel.sql.expressions import (Equals, NumericalLiteral,
                                       StringLiteral, And, Parameter)
+from squyrrel.sql.query_builder import QueryBuilder
 from squyrrel.sql.references import ColumnReference
 from squyrrel.sql.join import OnJoinCondition, JoinConstruct, JoinType
 
 
 class QueryWizzard:
 
-    def __init__(self, db: SqlDatabaseConnection, builder):
+    def __init__(self, db: SqlDatabaseConnection, builder=None):
         self.db = db
         self.builder = builder
         self.last_sql_query = None
@@ -27,11 +30,9 @@ class QueryWizzard:
         model_loaded_signal.connect(self.on_model_loaded)
 
     def commit(self):
-        # print('COMMIT')
         self.db.commit()
 
     def rollback(self):
-        # print('ROLLBACK')
         self.db.rollback()
 
     def last_insert_rowid(self):
@@ -99,7 +100,7 @@ class QueryWizzard:
                 return self.models[model]
             except KeyError:
                 models = ', '.join(self.models.keys())
-                raise Exception(f'Orm: did not find model {model}. Registered models are: {models}')
+                raise ModelNotFoundException(model, models)
         return model
 
     def get_model_by_table(self, table_name):
@@ -108,33 +109,10 @@ class QueryWizzard:
                 return model
         return None
 
-    def build_select_fields(self, model, select_fields=None):
-        if select_fields is None:
-            select_fields = []
-            for field_name, field in model.fields():
-                select_fields.append(ColumnReference(field_name, table=model.table_name))
-        return select_fields
-
-    def build_where_clause(self, model, filter_condition=None, **kwargs):
-        # todo: this is garbage
-        if filter_condition is None:
-            filter_conditions = []
-            for key, value in kwargs.items():
-                filter_conditions.append(
-                    Equals.column_as_parameter(ColumnReference(key, table=model.table_name), value))
-            if filter_conditions:
-                return WhereClause(filter_conditions[0])
-            else:
-                return None
-        else:
-            return WhereClause(filter_condition)
-
-    def m2m_aggregation_subquery_alias(self, model, relation_name):
-        return f'{model.table_name}_{relation_name}'
-
-    def build_m2m_aggregation_subquery(self, model, from_clause, relation_name, m2m_relation):
+    def build_m2m_aggregation_subquery(self, model, relation_name, m2m_relation):
+        # todo: refactor using QueryBuilder
         foreign_model = self.get_model(m2m_relation.foreign_model)
-        subquery_tablename = self.m2m_aggregation_subquery_alias(model, relation_name)
+        subquery_tablename = m2m_aggregation_subquery_alias(model, relation_name)
         select_fields = [ColumnReference(model.id_field_name(), alias=model.id_field_name()),
                          m2m_relation.aggregation]
         join_condition = OnJoinCondition(
@@ -148,7 +126,7 @@ class QueryWizzard:
             join_condition=join_condition
         )
         return Query(
-            select_clause=SelectClause.build(*select_fields),
+            select_clause=SelectClause(*select_fields),
             from_clause=from_clause,
             groupby_clause=GroupByClause(model.id_field_name()),
             is_subquery=True,
@@ -158,7 +136,7 @@ class QueryWizzard:
     def handle_many_to_one(self, model, select_fields, relation, from_clause):
         relation.foreign_model = self.get_model(relation.foreign_model)
         foreign_model = self.get_model(relation.foreign_model)
-        foreign_select_fields = self.build_select_fields(foreign_model)
+        foreign_select_fields = foreign_model.build_select_fields()
 
         # todo: make builder method specially for columns on OnJoinCondition
         join_condition = OnJoinCondition(
@@ -211,7 +189,6 @@ class QueryWizzard:
 
         aggr_subquery = self.build_m2m_aggregation_subquery(
             model=model,
-            from_clause=from_clause,
             relation_name=relation_name,
             m2m_relation=relation
         )
@@ -241,7 +218,7 @@ class QueryWizzard:
         aggr_select_fields = [ColumnReference(model.id_field_name(), alias=model.id_field_name()),
                               aggregation]
         subquery = Query(
-            select_clause=SelectClause.build(*aggr_select_fields),
+            select_clause=SelectClause(*aggr_select_fields),
             from_clause=FromClause(relation.foreign_model.table_name),
             groupby_clause=GroupByClause(model.id_field_name()),
             is_subquery=True,
@@ -261,93 +238,50 @@ class QueryWizzard:
         relation.table_name = subquery_tablename
         aggregations.append((relation_name, relation))
 
-    def load_many_to_many_entities(self, model, id, m2m_options, entity_format=EntityFormat.MODEL):
-        # todo: refactor: combine with load_one_to_many_entities
-        if m2m_options is None:
-            m2m_options = {'load_m2m': True}
-        else:
-            if not m2m_options.get('load_m2m', True):
-                return {}
+    def build_relation_to_many_query(self, model, instance_id, relation, options) -> Query:
+        orderby = None
+        page_size = None
+        active_page = None
+
+        if options is not None:
+            orderby = options.get('orderby', None)
+            page_size = options.get('page_size', None)
+            active_page = options.get('active_page', None)
+
+        filter_condition = Equals(ColumnReference(model.id_field_name(), table=model.table_name),
+                                  NumericalLiteral(instance_id))
+
+        return QueryBuilder(relation.foreign_model, self) \
+            .add_filter_condition(filter_condition) \
+            .orderby(orderby) \
+            .pagination(active_page, page_size) \
+            .build()
+
+    def load_relation_to_many_entities(self, model, instance_id, relations_dict, options):
+
+        if not options.get('load_entities', True):
+            return {}
 
         data = {}
-
-        for relation_name in model.many_to_many_dict():
-            relation = getattr(model, relation_name)
-
+        for relation_name in relations_dict:
+            relation = model.get_relation(relation_name)  # getattr(model, relation_name)
             if relation.lazy_load:
                 continue
-            filter_condition = Equals(ColumnReference(model.id_field_name(), table=model.table_name),
-                                      NumericalLiteral(id))
-
-            orderby = None
-            page_size = None
-            active_page = None
-
-            # todo: refactor, how to make sure, options is dict?
-            options = m2m_options.get(relation_name, None)
-            if options is not None:
-                dont_load = options.get('dont_load', False)
-                if dont_load:
-                    continue
-                orderby = options.get('orderby', None)
-                page_size = options.get('page_size', None)
-                active_page = options.get('active_page', None)
-
-            # print('handle relation:', relation_name)
-            data[relation_name] = self.get_all(relation.foreign_model,
-                                               filter_condition=filter_condition,
-                                               orderby=orderby,
-                                               page_size=page_size,
-                                               active_page=active_page,
-                                               entity_format=entity_format)
-            # print('entities:', relation.entities)
-        return data
-
-    def load_one_to_many_entities(self, model, id, one_to_many_options, entity_format=EntityFormat.MODEL):
-        # print('load_one_to_many_entities, options:')
-        # print(one_to_many_options)
-        if one_to_many_options is None:
-            one_to_many_options = {'load_12m': True}
-        else:
-            if not one_to_many_options.get('load_12m', True):
-                return
-
-        data = {}
-
-        for relation_name in model.one_to_many_dict():
-            relation = model.get_relation(relation_name)
-            if relation.lazy_load:
+            relation_options = options.get(relation_name, None)
+            if relation_options is not None and relation_options.get('skip', False):
                 continue
-
-            filter_condition = Equals(ColumnReference(model.id_field_name(), table=model.table_name),
-                                      NumericalLiteral(id))
-            orderby = None
-            page_size = None
-            active_page = None
-
-            options = one_to_many_options.get(relation_name, None)
-            if options is not None:
-                dont_load = options.get('dont_load', False)
-                if dont_load:
-                    continue
-                orderby = options.get('orderby', None)
-                page_size = options.get('page_size', None)
-                active_page = options.get('active_page', None)
-
-            # print('handle relation:', relation_name)
-            data[relation_name] = self.get_all(relation.foreign_model,
-                                               filter_condition=filter_condition,
-                                               orderby=orderby,
-                                               page_size=page_size,
-                                               active_page=active_page,
-                                               entity_format=entity_format)
-            # print(f'loaded {len(relation.entities)} entities')
+            query = self.build_relation_to_many_query(model, instance_id, relation, relation_options)
+            query.model = relation.foreign_model
+            data[relation_name] = self.get_all(query, entity_format=options.get('entity_format', EntityFormat.MODEL))
         return data
 
+    # todo: make this method to method of FromClause
     def include_many_to_many_join(self, model, relation, from_clause):
         # !! todo: first check if not already joined!!
 
-        foreign_model = self.get_model(relation.foreign_model)
+        # todo: refactor: compare with QueryBuilder._include_many_to_many_join
+
+        # foreign_model = self.get_model(relation.foreign_model)
         # foreign_select_fields = self.build_select_fields(foreign_model)
         junction_join_condition = OnJoinCondition(
             Equals(ColumnReference(model.id_field_name(), table=model.table_name),
@@ -359,23 +293,6 @@ class QueryWizzard:
             join_condition=junction_join_condition
         )
 
-    def build_get_all_query(self,
-                            model, select_fields=None, filter_condition=None, filters=None, fulltext_search=None,
-                            orderby=None, ascending=None, page_size=None, active_page=None):
-        """filters and filter_condition cannot be both not None"""
-
-        # print('\nbuild_get_all_query\n')
-        qb = QueryBuilder(model=self.get_model(model), qw=self)
-        return qb.build_get_all_query(
-            select_fields=select_fields,
-            filter_condition=filter_condition,
-            filters=filters,
-            fulltext_search=fulltext_search,
-            orderby=orderby,
-            ascending=ascending,
-            page_size=page_size,
-            active_page=active_page)
-
     def include_many_to_many_aggregations(self, model, from_clause, select_fields):
         m2m_aggregations = []
         for relation_name, relation in model.many_to_many_relations():
@@ -404,8 +321,8 @@ class QueryWizzard:
 
     def get_by_id(self, model, id, select_fields=None,
                   m2m_options=None, one_to_many_options=None,
-                  raise_ifnotfound=True, disable_relations=False,
-                  entity_format=EntityFormat.MODEL, **kwargs):
+                  raise_if_not_found=True, disable_relations=False,
+                  entity_format=EntityFormat.MODEL):
         model = self.get_model(model)
         filter_condition = Equals.id_as_parameter(model, id)
         # filter_condition = Equals(ColumnReference(model.id_field_name(), table=model.table_name),
@@ -416,67 +333,75 @@ class QueryWizzard:
                             m2m_options=m2m_options,
                             one_to_many_options=one_to_many_options,
                             disable_relations=disable_relations,
-                            entity_format=entity_format,
-                            **kwargs)
-        if instance is None and raise_ifnotfound:
+                            entity_format=entity_format)
+        if instance is None and raise_if_not_found:
             raise DidNotFindObjectWithIdException(
                 msg=f'Did not find {model.__name__} with id {id}',
                 model_name=model.__name__,
                 id=id)
         return instance
 
-    def get(self, model, select_fields=None,
-            filter_condition=None, m2m_options=None,
-            one_to_many_options=None, disable_relations=False,
-            entity_format=EntityFormat.MODEL, **kwargs):
+    def get(self,
+            model: Type[Model],
+            select_fields=None,
+            filter_condition=None,
+            m2m_options=None,
+            one_to_many_options=None,
+            disable_relations=False,
+            entity_format=EntityFormat.MODEL):
 
-        model = self.get_model(model)
-        select_fields = self.build_select_fields(model, select_fields)
-        where_clause = self.build_where_clause(model, filter_condition=filter_condition, **kwargs)
-        from_clause = FromClause(model.table_name)
+        if select_fields is None:
+            select_fields = model.build_select_fields()
 
+        query = QueryBuilder(model, self) \
+            .select(select_fields) \
+            .add_filter_condition(filter_condition) \
+            .build()
+
+        # todo: move blow to next if not disable_relations...
         if disable_relations:
             one_to_many_aggregations = []
             m2m_aggregations = []
         else:
-            one_to_many_aggregations = self.include_one_to_many_aggregations(
-                model=model, from_clause=from_clause, select_fields=select_fields)
-            m2m_aggregations = self.include_many_to_many_aggregations(
-                model=model, from_clause=from_clause, select_fields=select_fields)
+            one_to_many_aggregations = self.include_one_to_many_aggregations(model=model,
+                                                                             from_clause=query.from_clause,
+                                                                             select_fields=select_fields)
+            m2m_aggregations = self.include_many_to_many_aggregations(model=model,
+                                                                      from_clause=query.from_clause,
+                                                                      select_fields=select_fields)
 
-        many_to_one_entities = self.handle_many_to_one_entities(
-            model=model, select_fields=select_fields, from_clause=from_clause)
-
-        query = Query(
-            select_clause=SelectClause.build(*select_fields),
-            from_clause=from_clause,
-            where_clause=where_clause,
-            pagination=None
-        )
+        many_to_one_entities = self.handle_many_to_one_entities(model=model,
+                                                                select_fields=select_fields,
+                                                                from_clause=query.from_clause)
 
         self.execute_query(query)
         data = self.db.fetchone()
 
         if data is None:
             return None
+
         data = self.model_instance_dict(model, data, entity_format, select_fields,
                                         many_to_one_entities, one_to_many_aggregations,
                                         m2m_aggregations)
         entity_id = data.get(model.id_field_name())
 
+        # print(data)
+
         if not disable_relations:
-            m2m_data = self.load_many_to_many_entities(
-                model,
-                id=entity_id,
-                m2m_options=m2m_options,
-                entity_format=entity_format)
-            data.update(**m2m_data)
-            one_to_many_data = self.load_one_to_many_entities(
-                model,
-                id=entity_id,
-                one_to_many_options=one_to_many_options,
-                entity_format=entity_format)
-            data.update(**one_to_many_data)
+            data.update(
+                **self.load_relation_to_many_entities(model,
+                                                      instance_id=entity_id,
+                                                      relations_dict=model.many_to_many_dict(),
+                                                      options={**(m2m_options or {}),
+                                                               'entity_format': entity_format})
+            )
+            data.update(
+                **self.load_relation_to_many_entities(model,
+                                                      instance_id=entity_id,
+                                                      relations_dict=model.one_to_many_dict(),
+                                                      options={**(one_to_many_options or {}),
+                                                               'entity_format': entity_format})
+            )
 
         # todo: refactor in method/class entitybuilder
         if entity_format == EntityFormat.MODEL:
@@ -494,7 +419,6 @@ class QueryWizzard:
     def load_filter_values(self, filters):
         if filters is None:
             return
-        kwargs = {}
         for filter_ in filters:
             if isinstance(filter_, (ManyToOneFilter, ManyToManyFilter)):
                 filter_.entities = list()
@@ -505,28 +429,21 @@ class QueryWizzard:
                                            id=id_value, disable_relations=True)
                         )
 
-    def get_all(self, model, select_fields=None, filter_condition=None, filters=None, fulltext_search=None,
-                orderby=None, ascending=None, page_size=None, active_page=None, include_count=False,
+    def get_all(self, query: Query, include_count=False,
                 entity_format=EntityFormat.MODEL):
-
-        query = self.build_get_all_query(model,
-                                         select_fields=select_fields,
-                                         filter_condition=filter_condition,
-                                         filters=filters, fulltext_search=fulltext_search,
-                                         orderby=orderby, ascending=ascending,
-                                         page_size=page_size, active_page=active_page)
-        model = self.get_model(model)
+        # todo: put include_count and entity_format into options
+        # todo: somehow manage query.model better: possibly as extra parameter of this method instead of attribute of query
 
         from_clause = query.from_clause
         select_fields = query.select_clause.items
 
         # war vorher vor filter_condition is not None..
         many_to_one_entities = self.handle_many_to_one_entities(
-            model=model, select_fields=select_fields, from_clause=from_clause)
+            model=query.model, select_fields=select_fields, from_clause=from_clause)
         one_to_many_aggregations = self.include_one_to_many_aggregations(
-            model=model, from_clause=from_clause, select_fields=select_fields)
+            model=query.model, from_clause=from_clause, select_fields=select_fields)
         m2m_aggregations = self.include_many_to_many_aggregations(
-            model=model, from_clause=from_clause, select_fields=select_fields)
+            model=query.model, from_clause=from_clause, select_fields=select_fields)
 
         self.db.create_cursor()
         self.execute_query(query)
@@ -534,7 +451,7 @@ class QueryWizzard:
         res = self.db.fetchall()
         return self.build_get_all_response(res=res, include_count=include_count, query=query,
                                            entity_format=entity_format,
-                                           model=model, select_fields=select_fields,
+                                           model=query.model, select_fields=select_fields,
                                            many_to_one_entities=many_to_one_entities,
                                            one_to_many_aggregations=one_to_many_aggregations,
                                            m2m_aggregations=m2m_aggregations)
@@ -551,9 +468,14 @@ class QueryWizzard:
         entities = []
         if entity_format == EntityFormat.MODEL:
             for r in res:
-                data = self.model_instance_dict(model, r, entity_format, select_fields,
-                                                many_to_one_entities, one_to_many_aggregations,
-                                                m2m_aggregations)
+                data = self.model_instance_dict(
+                    model,
+                    r,
+                    entity_format,
+                    select_fields,
+                    many_to_one_entities,
+                    one_to_many_aggregations,
+                    m2m_aggregations)
                 entities.append(
                     self.build_entity(
                         model,
@@ -568,7 +490,6 @@ class QueryWizzard:
                                              many_to_one_entities, one_to_many_aggregations,
                                              m2m_aggregations)
                 )
-        print("include count: ", include_count)
         if include_count:
             count = self.count(model, query=query)
             return {'entities': entities, 'count': count}
@@ -587,6 +508,7 @@ class QueryWizzard:
                 first_result = results[0]
                 relation.aggregation_value = first_result[1]
 
+    # todo: make to instance or class method of Model?
     def add_congregation_values_to_entity(self, entity, data, select_fields):
         congregate_fields = entity.congregate_fields()
         for congregate_field_name, congregate_field in congregate_fields:
@@ -639,6 +561,7 @@ class QueryWizzard:
         self.add_congregation_values_to_entity(entity, data, select_fields)
         return entity
 
+    # todo: make static or utility
     def get_data(self, data, select_fields, reference):
         results = []
         # print('get_Data, reference:', reference)
@@ -659,10 +582,11 @@ class QueryWizzard:
         relation = getattr(entity, relation_name)
 
         filter_condition = Equals.id_as_parameter(model, entity.id)
-        query = self.build_get_all_query(model, select_fields=[f'count (*)'],
-                                         filter_condition=filter_condition, orderby=None,
-                                         page_size=None, active_page=None)
-        query.orderby_clause = None
+        query = QueryBuilder(model, self) \
+            .select([f'count (*)']) \
+            .add_filter_condition(filter_condition) \
+            .orderby(None) \
+            .build()
 
         self.include_many_to_many_join(model, relation, query.from_clause)
 
@@ -677,19 +601,17 @@ class QueryWizzard:
         if query is not None:
             # or count(*)
             select_fields = [f'count ({ColumnReference(model.id_field_name(), table=model.table_name)})']
-            query.select_clause = SelectClause.build(*select_fields)
+            query.select_clause = SelectClause(*select_fields)
             query.pagination = None
             # query.select_clause.select_fields = [f'count ({ColumnReference(model.id_field_name(), table=model.table_name)})']
         else:
-            query = self.build_get_all_query(model,
-                                             select_fields=[
-                                                 f'count ({ColumnReference(model.id_field_name(), table=model.table_name)})'],
-                                             filter_condition=filter_condition,
-                                             filters=filters,
-                                             fulltext_search=fulltext_search,
-                                             orderby=None,
-                                             page_size=None,
-                                             active_page=None)
+            query = QueryBuilder(model, self) \
+                .select([f'count ({ColumnReference(model.id_field_name(), table=model.table_name)})']) \
+                .add_filter_condition(filter_condition) \
+                .model_filters(filters) \
+                .fulltext_search(fulltext_search) \
+                .build()
+
         query.orderby_clause = None
 
         self.execute_query(query)
@@ -709,16 +631,13 @@ class QueryWizzard:
         # todo: replace with parameter builder method
         filter_condition = Equals(ColumnReference(search_column, table=model.table_name),
                                   literal)
-        where_clause = self.build_where_clause(model, filter_condition=filter_condition)
 
-        query = Query(
-            select_clause=SelectClause.build(*select_fields),
-            from_clause=FromClause(model.table_name),
-            where_clause=where_clause,
-            pagination=None
-        )
-        return query
+        return QueryBuilder(model, self) \
+            .select(select_fields) \
+            .add_filter_condition(filter_condition) \
+            .build()
 
+    # todo: make static
     def prepare_m2m_data(self, model, prepared_data, instance=None):
         for m2m_relation_name, m2m_relation in model.many_to_many_relations():
             prepared_data[m2m_relation_name] = getattr(instance, m2m_relation_name).entities
@@ -734,14 +653,15 @@ class QueryWizzard:
                 pass
             else:
                 if isinstance(relation, ManyToOne):
-                    print('handle m21:', relation_name)
+                    # print('handle m21:', relation_name)
                     # if columns not equal
                     # refactor: retrieve value by id
                     relation_foreign_model = self.get_model(relation.foreign_model)
                     if relation.load_all:
-                        print('load_all')
-                        prepared_data[relation_name + '_all'] = self.get_all(relation_foreign_model)
-                        print(prepared_data[relation_name + '_all'])
+                        # print('load_all')
+                        query = QueryBuilder(relation_foreign_model, self).build()
+                        prepared_data[relation_name + '_all'] = self.get_all(query)
+                        # print(prepared_data[relation_name + '_all'])
 
                     prepared_value = self.retrieve_value_by_value(
                         model=relation_foreign_model,
@@ -781,24 +701,23 @@ class QueryWizzard:
             filter_value=filter_value
         )
         if id_ is None:
-            raise DidNotFindForeignIdException(f'Did not find {model.__name__} with {filter_column} = {filter_value}',
-                                               field=filter_column)
+            raise DidNotFindForeignIdException(
+                f'Did not find {model.__name__} with {filter_column} = {filter_value}',
+                field=filter_column)
         return id_
 
     def fulltext_search(self, model, search_value, pagesize, active_page, include_count=True, json=False):
         data = {}
 
-        qb = QueryBuilder(model=self.get_model(model), qw=self)
-        search_condition = qb.build_search_condition(search_value=search_value)
-        # todo: refactor: bring both qw-methods together..
-        data['list'] = self.get_all(
-            model=model,
-            page_size=pagesize,
-            active_page=active_page,
-            filter_condition=search_condition
-        )
+        query = QueryBuilder(model, self) \
+            .fulltext_search(search_value) \
+            .pagination(active_page, page_size=pagesize) \
+            .build()
+
+        data['list'] = self.get_all(query)
+
         if include_count:
-            data['count'] = self.count(model=model, filter_condition=search_condition)
+            data['count'] = self.count(model=model, query=query)
         if json:
             data['list'] = [el.as_json() for el in data['list']]
         return data
@@ -942,9 +861,6 @@ class QueryWizzard:
                 updates[column] = value
                 continue
 
-            if not value:
-                continue
-
             relation = model.get_relation(column)
 
             # todo: refactor
@@ -985,19 +901,24 @@ class QueryWizzard:
         model = self.get_model(model)
         filter_condition = Equals.id_as_parameter(model, instance_id)
         # todo: add logging
-        print("data:")
-        print(data)
         self.update(model=model, filter_condition=filter_condition, data=data, prev_data=prev_data,
                     instance_id=instance_id, commit=commit)
         if return_updated_object:
-            a = self.get_by_id(model, instance_id, entity_format=EntityFormat.JSON)
-            print('update_by_id')
-            print(a)
-            return a
+            return self.get_by_id(model, instance_id, entity_format=EntityFormat.JSON)
+
+    def field_to_sql_data_type(self, field):
+        # todo: dynamic method_name pattern
+        # at the moment only Sqlite...
+
+        if isinstance(field, StringField):
+            return 'TEXT'
+        if isinstance(field, IntegerField):
+            return 'INTEGER'
+        if isinstance(field, DateTimeField):
+            return 'TEXT'
 
     def build_create_table_query(self, model, if_not_exists=False):
         model = self.get_model(model)
-
         columns = {}
         for field_name, field in model.fields():
             columns[field_name] = {
@@ -1010,17 +931,7 @@ class QueryWizzard:
         return query
 
     def create_table(self, model, if_not_exists=False):
+        model = self.get_model(model)
         query = self.build_create_table_query(model, if_not_exists=if_not_exists)
         self.execute_query(query)
-        # todo: commit missingß
-
-    def field_to_sql_data_type(self, field):
-        # todo: dynamic method_name pattern
-        # at the moment only Sqlite...
-
-        if isinstance(field, StringField):
-            return 'TEXT'
-        if isinstance(field, IntegerField):
-            return 'INTEGER'
-        if isinstance(field, DateTimeField):
-            return 'TEXT'
+        # todo: commit missing
